import pandas as pd
import numpy as np
import matplotlib
matplotlib.use('AGG')
import matplotlib.pyplot as plt

from matplotlib.pyplot import MultipleLocator

num_graphs = 6
num_cols = 6
num_rows = (num_graphs + num_cols - 1) // num_cols
plt.rcParams['xtick.labelsize'] = 15
plt.rcParams['ytick.labelsize'] = 15
f, ax1 = plt.subplots(num_rows, num_cols, figsize=(23, 3.5))
f.subplots_adjust(hspace=0.35, wspace=0.1)
plt.rc('pdf', fonttype=42)
plt.rcParams.update({'font.size': 15})
plt.rcParams['hatch.linewidth'] = 1  # previous pdf hatch linewidth
# plt.xticks(fontsize=12)
# plt.yticks(fontsize=12)

models = [
    'opt-13-finance',
    'opt-13-chatbot',
    'opt-13-dialogue',
    'llama2-70-finance',
    'llama2-70-chatbot',
    'llama2-70-dialogue'
]

models_name = [
    'OPT-13B-finance',
    'OPT-13B-chatbot',
    'OPT-13B-dialogue',
    'Llama2-70B-chat-finance',
    'llama2-70B-chat-chatbot',
    'llama2-70B-chat-dialogue'
]

width = 0.7


my_style = [
    {
        'color': '#fdd0a2',
        'hatch': '',
    },
    {
        'color': '#fdd0a2',
        'hatch': '',
    },
    {
        'color': '#fdae6b',
        'hatch': '',
    },
    {
        'color': '#ed7d31',
        'hatch': '',
    },
    {
        'color': '#eff3ff',
        'hatch': '\\',
    },
    {
        'color': '#c6dbef',
        'hatch': '',
    },
    {
        'color': '#9ecae1',
        'hatch': '',
    },
]

y_major_locator = MultipleLocator(0.5)
for idx, model in enumerate(models):
    idx_x = idx // num_cols
    idx_y = idx % num_cols

    variant_ids = ['throughput']
    variant_names = ['throughput']

    df = pd.read_excel('ablation_1.xlsx', sheet_name=model)
    # for id in variant_ids:
    #     series = df[id]
    #     for idx in range(len(series)):
    #         df[id][idx] = math.sqrt(df[id][idx])

    time = {g_name: df[g_id] for g_id, g_name in zip(variant_ids, variant_names)}
    groups = df['bs']

    ax = ax1
    if num_rows > 1:
        ax = ax[idx_x]
    if num_cols > 1:
        ax = ax[idx_y]

    ax.set_xlim(-1, len(df))
    # ax1[idx_x][idx_y].set_yscale('log')
    # ax1[0][0].set_ylim(0, 9)

    num_variants = len(variant_ids)

    xticks = [i for i in range(0, len(df))]
    xticks_bars = []
    if num_variants % 2 == 0:
        for x_offset in range(-(num_variants - 1) // 2, (num_variants // 2) + 1):
            xticks_bars.append(
                [i + (x_offset + .5)*width/num_variants for i in range(len(df))])
    else:
        for x_offset in range(-(num_variants - 1) // 2, (num_variants // 2) + 1):
            xticks_bars.append(
                [i + x_offset*width/num_variants for i in range(len(df))])

    for bar_id in range(num_variants):
        bar = ax.bar(
            xticks_bars[bar_id],
            time[variant_names[bar_id]],
            width=width/num_variants,
            label=variant_names[bar_id],
            edgecolor='black',
            linewidth=1,
            **my_style[bar_id],
        )

    ax.axhline(y=df['max'][0], color='red', ls='--', lw=2)
    ax.set_xticks(xticks)
    ax.set_xticklabels(groups)

    ax.set_title(f'({chr(ord("a") + idx)}) {models_name[idx]}', y=0, pad=-60)
    if idx == 0:
        ax.set_ylabel('Throughput (request/s)',fontsize=15)
    handles, labels = ax.get_legend_handles_labels()        
    # f.legend(handles, labels, loc='upper center', ncol=7,bbox_to_anchor=(0.5,1.05))


plt.tight_layout()
plt.show()
f.savefig('fig/ablation_1.pdf', bbox_inches='tight')
